@@ -1,5 +1,7 @@
+from io import BytesIO
+
 try:
-    from pillow_heif import AvifImagePlugin, HeifImagePlugin  # noqa: F401
+    from pillow_heif import HeifImagePlugin  # noqa: F401
 except ImportError:
     pass
 
@@ -9,6 +11,7 @@ from willow.image import (
     BMPImageFile,
     GIFImageFile,
     HeicImageFile,
+    IcoImageFile,
     Image,
     JPEGImageFile,
     PNGImageFile,
@@ -29,6 +32,12 @@ def _PIL_Image():
     return PIL.Image
 
 
+def _PIL_ImageCms():
+    import PIL.ImageCms
+
+    return PIL.ImageCms
+
+
 class PillowImage(Image):
     def __init__(self, image):
         self.image = image
@@ -166,6 +175,55 @@ class PillowImage(Image):
 
         return PillowImage(new_image.convert("RGB"))
 
+    def get_icc_profile(self):
+        return self.image.info.get("icc_profile")
+
+    def get_exif_data(self):
+        return self.image.info.get("exif")
+
+    @Image.operation
+    def transform_colorspace_to_srgb(self, rendering_intent=0):
+        """
+        Transforms the color of the image to fit inside sRGB color gamut using the
+        embedded ICC profile. The resulting image will always be in RGB(A) mode
+        and will have a small generic sRGB ICC profile embedded.
+
+        If the image does not have an ICC profile this operation is a no-op.
+        Images without a profile are commonly assumed to be in sRGB color space
+        already.
+
+        :param rendering_intent: Controls how out-of-gamut colors and handled.
+        Defaults to 0 (perceptual) because this is what Pillow defaults to.
+        :return: PillowImage in RGB mode
+        :raises: PIL.ImageCms.PyCMSError
+
+        Further reading:
+            * https://pillow.readthedocs.io/en/stable/reference/ImageCms.html#PIL.ImageCms.profileToProfile
+            * https://www.permajet.com/blog/rendering-intents-explained/
+        """
+        icc_profile = self.get_icc_profile()
+
+        # Can't transform if there is no profile, no-op
+        if icc_profile is None:
+            return self
+
+        ImageCms = _PIL_ImageCms()
+        # ImageCmsProfile expects profile data to be file-like, give it BytesIO that quacks like a file 🦆
+        icc_profile = ImageCms.ImageCmsProfile(BytesIO(icc_profile))
+
+        # Output mode should always be RGB, unless the image has an alpha channel.
+        output_mode = "RGBA" if self.has_alpha() else "RGB"
+
+        # Attempt to convert from the embedded profile of the image to a generic sRGB one
+        image = ImageCms.profileToProfile(
+            self.image,
+            icc_profile,
+            ImageCms.createProfile("sRGB"),
+            renderingIntent=rendering_intent,
+            outputMode=output_mode,
+        )
+        return PillowImage(image)
+
     @Image.operation
     def save_as_jpeg(
         self,
@@ -197,6 +255,14 @@ class PillowImage(Image):
         if progressive:
             kwargs["progressive"] = True
 
+        icc_profile = self.get_icc_profile()
+        if icc_profile is not None:
+            kwargs["icc_profile"] = icc_profile
+
+        exif_data = self.get_exif_data()
+        if exif_data is not None:
+            kwargs["exif"] = exif_data
+
         image.save(f, "JPEG", **kwargs)
         if apply_optimizers:
             self.optimize(f, "jpeg")
@@ -213,15 +279,43 @@ class PillowImage(Image):
         :param apply_optimizers: controls whether to run any configured optimizer libraries
         :return: PNGImageFile
         """
-        if self.image.mode == "CMYK":
-            image = self.image.convert("RGB")
-        else:
-            image = self.image
-        # Pillow only checks presence of optimize kwarg, not its value
+
         kwargs = {}
+        image = self.image
+        icc_profile = self.get_icc_profile()
+        if icc_profile is not None:
+            # If the image is in CMYK mode *and* has an ICC profile, we need to be more diligent
+            # about how we handle the color conversion to RGB. We don't want to retain
+            # the color profile as-is because it is not meant for RGB images and
+            # will result in inaccurate colors. The transformation to sRGB should result
+            # in a more accurate representation of the original image, though
+            # it will likely not be perfect.
+            if self.image.mode == "CMYK":
+                pillow_image = self.transform_colorspace_to_srgb()
+                image = pillow_image.image
+                kwargs["icc_profile"] = pillow_image.get_icc_profile()
+            else:
+                kwargs["icc_profile"] = icc_profile
+
+        elif image.mode == "CMYK":
+            image = image.convert("RGB")
+
+        # Pillow 11.2 and older used to silently clip 32-bit I-mode images to 16
+        # bits, which is the maximum PNG supports. This is silent behavior
+        # deprecated in Pillow 11.3 but we explicitly want to retain it for
+        # backwards compatibility.
+        # See: https://pillow.readthedocs.io/en/stable/releasenotes/11.3.0.html#saving-i-mode-images-as-png
+        if image.mode == "I":
+            image = image.convert("I;16")
+
+        # Pillow only checks presence of optimize kwarg, not its value
         if optimize:
             kwargs["optimize"] = True
 
+        exif_data = self.get_exif_data()
+        if exif_data is not None:
+            kwargs["exif"] = exif_data
+
         image.save(f, "PNG", **kwargs)
         if apply_optimizers:
             self.optimize(f, "png")
@@ -264,7 +358,27 @@ class PillowImage(Image):
             Note that when lossless=True, this will be ignored.
         :return: WebPImageFile
         """
-        self.image.save(f, "WEBP", quality=quality, lossless=lossless)
+
+        kwargs = {"quality": quality, "lossless": lossless}
+
+        image = self.image
+        icc_profile = self.get_icc_profile()
+        if icc_profile is not None:
+            # If the image is in CMYK mode *and* has an ICC profile, we need to be more diligent
+            # about how we handle the color space. WEBP will encode as RGB so we need to do extra
+            # work to ensure the colors are as accurate as possible. We don't want to retain
+            # the color profile as-is because it is not meant for RGB images and
+            # will result in inaccurate colors. The transformation to sRGB should result
+            # in a more accurate representation of the original image, though
+            # it will likely not be perfect.
+            if image.mode == "CMYK":
+                pillow_image = self.transform_colorspace_to_srgb()
+                image = pillow_image.image
+                kwargs["icc_profile"] = pillow_image.get_icc_profile()
+            else:
+                kwargs["icc_profile"] = icc_profile
+
+        image.save(f, "WEBP", **kwargs)
         if apply_optimizers and not lossless:
             self.optimize(f, "webp")
         return WebPImageFile(f)
@@ -287,63 +401,85 @@ class PillowImage(Image):
             Note that when lossless=True, this will be ignored.
         :return: HeicImageFile
         """
+        kwargs = {"quality": quality}
         if lossless:
-            self.image.save(f, "HEIF", quality=-1, chroma=444)
-        else:
-            self.image.save(f, "HEIF", quality=quality)
-            if apply_optimizers:
-                self.optimize(f, "heic")
+            kwargs = {"quality": -1, "chroma": 444}
+
+        image = self.image
+        icc_profile = self.get_icc_profile()
+        if icc_profile is not None:
+            # If the image is in CMYK mode *and* has an ICC profile, we need to be more diligent
+            # about how we handle the color space. HEIC will encode as RGB so we need to do extra
+            # work to ensure the colors are as accurate as possible. We don't want to retain
+            # the color profile as-is because it is not meant for RGB images and
+            # will result in inaccurate colors. The transformation to sRGB should result
+            # in a more accurate representation of the original image, though
+            # it will likely not be perfect.
+            if image.mode == "CMYK":
+                pillow_image = self.transform_colorspace_to_srgb()
+                image = pillow_image.image
+                kwargs["icc_profile"] = pillow_image.get_icc_profile()
+            else:
+                kwargs["icc_profile"] = icc_profile
+
+        image.save(f, "HEIF", **kwargs)
+
+        if not lossless and apply_optimizers:
+            self.optimize(f, "heic")
 
         return HeicImageFile(f)
 
     @Image.operation
     def save_as_avif(self, f, quality=80, lossless=False, apply_optimizers=True):
+        kwargs = {"quality": quality}
         if lossless:
-            self.image.save(f, "AVIF", quality=-1, chroma=444)
-        else:
-            self.image.save(f, "AVIF", quality=quality)
-            if apply_optimizers:
-                self.optimize(f, "heic")
+            kwargs = {
+                # Quality of 100 implies lossless (according to libavif documentation)
+                "quality": 100,
+                # When encoding lossless images, don't use chroma subsampling (4:4:4 retains the most information)
+                "subsampling": "4:4:4",
+            }
+
+        image = self.image
+        icc_profile = self.get_icc_profile()
+        if icc_profile is not None:
+            # If the image is in CMYK mode *and* has an ICC profile, we need to be more diligent
+            # about how we handle the color space. AVIF will encode as RGB so we need to do extra
+            # work to ensure the colors are as accurate as possible. We don't want to retain
+            # the color profile as-is because it is not meant for RGB images and
+            # will result in inaccurate colors. The transformation to sRGB should result
+            # in a more accurate representation of the original image, though
+            # it will likely not be perfect.
+            if image.mode == "CMYK":
+                pillow_image = self.transform_colorspace_to_srgb()
+                image = pillow_image.image
+                kwargs["icc_profile"] = pillow_image.get_icc_profile()
+            else:
+                kwargs["icc_profile"] = icc_profile
+
+        image.save(f, "AVIF", **kwargs)
+
+        if not lossless and apply_optimizers:
+            self.optimize(f, "heic")
 
         return AvifImageFile(f)
 
+    @Image.operation
+    def save_as_ico(self, f, apply_optimizers=True):
+        self.image.save(f, "ICO")
+
+        if apply_optimizers:
+            self.optimize(f, "ico")
+
+        return IcoImageFile(f)
+
     @Image.operation
     def auto_orient(self):
         # JPEG files can be orientated using an EXIF tag.
         # Make sure this orientation is applied to the data
-        image = self.image
+        from PIL import ImageOps
 
-        if hasattr(image, "_getexif"):
-            try:
-                exif = image._getexif()
-            except Exception:  # noqa: BLE001
-                # Blanket cover all the ways _getexif can fail in.
-                exif = None
-            if exif is not None:
-                # 0x0112 = Orientation
-                orientation = exif.get(0x0112, 1)
-
-                if 1 <= orientation <= 8:
-                    Image = _PIL_Image()
-                    ORIENTATION_TO_TRANSPOSE = {
-                        1: (),
-                        2: (Image.Transpose.FLIP_LEFT_RIGHT,),
-                        3: (Image.Transpose.ROTATE_180,),
-                        4: (
-                            Image.Transpose.ROTATE_180,
-                            Image.Transpose.FLIP_LEFT_RIGHT,
-                        ),
-                        5: (
-                            Image.Transpose.ROTATE_270,
-                            Image.Transpose.FLIP_LEFT_RIGHT,
-                        ),
-                        6: (Image.Transpose.ROTATE_270,),
-                        7: (Image.Transpose.ROTATE_90, Image.Transpose.FLIP_LEFT_RIGHT),
-                        8: (Image.Transpose.ROTATE_90,),
-                    }
-
-                    for transpose in ORIENTATION_TO_TRANSPOSE[orientation]:
-                        image = image.transpose(transpose)
+        image = ImageOps.exif_transpose(self.image)
 
         return PillowImage(image)
 
@@ -360,6 +496,7 @@ class PillowImage(Image):
     @Image.converter_from(WebPImageFile)
     @Image.converter_from(HeicImageFile)
     @Image.converter_from(AvifImageFile)
+    @Image.converter_from(IcoImageFile)
     def open(cls, image_file):
         image_file.f.seek(0)
         image = _PIL_Image().open(image_file.f)
